@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -537,7 +534,7 @@ about_destroy (GObject *obj, GObject *deadbeef)
 }
 
 static const char *authors[] = {
-	"Jeffrey Stedfast <fejj@ximian.com>",
+	"Jeffrey Stedfast <fejj@novell.com>",
 	NULL
 };
 
@@ -551,7 +548,7 @@ help_about_cb (GtkWidget *widget, gpointer user_data)
 	
 	grind = (Alleyoop *) user_data;
 	if (grind->about == NULL) {
-		grind->about = gnome_about_new ("Alleyoop", VERSION, "Copyright 2003 Jeffrey Stedfast <fejj@ximian.com>",
+		grind->about = gnome_about_new ("Alleyoop", VERSION, "Copyright 2003-2006 Jeffrey Stedfast <fejj@novell.com>",
 						_("Alleyoop is a Valgrind front-end for the GNOME environment."),
 						(const char **) authors, NULL, _(translation), NULL);
 		
@@ -791,9 +788,10 @@ alleyoop_run (Alleyoop *grind, GError **err)
 	
 	args = alleyoop_prefs_create_argv ((AlleyoopPrefs *) grind->prefs, grind->tool);
 	
-	sprintf (logfd_arg, "--logfile-fd=%d", logfd[1]);
+	sprintf (logfd_arg, "--log-fd=%d", logfd[1]);
 	g_ptr_array_add (args, logfd_arg);
 	
+	g_ptr_array_add (args, "--");
 	for (i = 0; grind->argv[i] != NULL; i++)
 		g_ptr_array_add (args, (char *) grind->argv[i]);
 	g_ptr_array_add (args, NULL);
