@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -31,31 +28,15 @@
 #include <sys/stat.h>
 #include <dirent.h>
 
-#include <popt.h>
 #include <gnome.h>
 #include <glade/glade.h>
 #include <gconf/gconf.h>
 
+#include "getopts.h"
 #include "alleyoop.h"
 #include "vgstrpool.h"
 
 
-#ifndef POPT_TABLEEND
-#define POPT_TABLEEND { NULL, '\0', 0, NULL, '\0', NULL, NULL }
-#endif
-
-static const struct poptOption options[] = {
-	{ "version", 'v', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH, NULL, 'v',
-	  N_("Print version and exit"), NULL },
-	{ "include", 'I', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH, NULL, 'I',
-	  N_("Add <dir> to the list of directories to search for source files"), "<dir>" },
-	{ "recursive", 'R', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH, NULL, 'R',
-	  N_("Recursively add <dir> and all subdirectories to the list of directories to search for source files"), "<dir>" },
-	{ N_("tool"), '\0', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH, NULL, 's',
-	  N_("Specify the default Valgrind tool to use"), "<name>" },
-	POPT_TABLEEND
-};
-
 static gboolean
 alleyoop_close (GtkWidget *widget, gpointer user_data)
 {
@@ -103,18 +84,106 @@ add_subdirs (GPtrArray *srcdir, GPtrArray *gc, const char *topsrcdir)
 	g_ptr_array_free (subdirs, TRUE);
 }
 
+
+static const char *tool = NULL;
+static GPtrArray *srcdir, *gc;
+
+static void
+shutdown (void)
+{
+	size_t i;
+	
+	for (i = 0; i < gc->len; i++)
+		g_free (gc->pdata[i]);
+	g_ptr_array_free (gc, TRUE);
+	
+	g_ptr_array_free (srcdir, TRUE);
+}
+
+static int
+display_help (GetOptsContext *ctx, GetOptsOption *opt, const char *arg, void *valuep)
+{
+	printf ("Usage: %s [OPTIONS...] [program [options]]\n\n", PACKAGE);
+	getopts_print_help (ctx);
+	shutdown ();
+	exit (0);
+	
+	return 0;
+}
+
+static int
+display_version (GetOptsContext *ctx, GetOptsOption *opt, const char *arg, void *valuep)
+{
+	fputs (PACKAGE " " VERSION "\n", stdout);
+	shutdown ();
+	exit (0);
+	
+	return 0;
+}
+
+static int
+include_dir (GetOptsContext *ctx, GetOptsOption *opt, const char *arg, void *valuep)
+{
+	g_ptr_array_add (srcdir, (char *) arg);
+	
+	return 0;
+}
+
+static int
+recurse_dir (GetOptsContext *ctx, GetOptsOption *opt, const char *arg, void *valuep)
+{
+	g_ptr_array_add (srcdir, (char *) arg);
+	add_subdirs (srcdir, gc, arg);
+	
+	return 0;
+}
+
+static int
+use_tool (GetOptsContext *ctx, GetOptsOption *opt, const char *arg, void *valuep)
+{
+	if (!g_ascii_strcasecmp (arg, "memcheck")) {
+		/* default */
+		tool = NULL;
+	} else if (!g_ascii_strcasecmp (arg, "addrcheck")) {
+		tool = "addrcheck";
+	} else if (!g_ascii_strcasecmp (arg, "cachegrind")) {
+		/*tool = "cachegrind";*/
+		fprintf (stderr, "%s is currently an unsupported tool\n", arg);
+	} else if (!g_ascii_strcasecmp (arg, "helgrind")) {
+		/*tool = "helgrind";*/
+		fprintf (stderr, "%s is currently an unsupported tool\n", arg);
+	} else {
+		fprintf (stderr, "Unknown tool: %s\n", arg);
+		shutdown ();
+		exit (0);
+	}
+	
+	return 0;
+}
+
+#define HELP_TEXT      N_("Display help and quit")
+#define VERSION_TEXT   N_("Display version and quit")
+#define INCLUDE_TEXT   N_("Add <dir> to the list of directories to search for source files")
+#define RECURSIVE_TEXT N_("Recursively add <dir> and all subdirectories to the list of directories to search for source files")
+#define TOOL_TEXT      N_("Specify the default Valgrind tool to use")
+
+static GetOptsOption options[] = {
+	{ "help",      'h',  GETOPTS_NO_ARG | GETOPTS_ARG_CUSTOM, HELP_TEXT,      NULL,     'h', display_help,    NULL },
+	{ "version",   'v',  GETOPTS_NO_ARG | GETOPTS_ARG_CUSTOM, VERSION_TEXT,   NULL,     'v', display_version, NULL },
+	{ "include",   'I',  GETOPTS_REQUIRED_STRING_ARG,         INCLUDE_TEXT,   "<dir>",  'I', include_dir,     NULL },
+	{ "recursive", 'R',  GETOPTS_REQUIRED_STRING_ARG,         RECURSIVE_TEXT, "<dir>",  'R', recurse_dir,     NULL },
+	{ "tool",      '\0', GETOPTS_REQUIRED_STRING_ARG,         TOOL_TEXT,      "<name>", 't', use_tool,        NULL },
+	GETOPTS_TABLE_END
+};
+
 int main (int argc, char **argv)
 {
-	const char *tool = NULL;
-	GPtrArray *srcdir, *gc;
 	GnomeProgram *program;
+	GetOptsContext *ctx;
 	GtkWidget *alleyoop;
 	const char **args;
-	const char *path;
-	const char *name;
 	char *srcdir_env;
-	poptContext ctx;
-	int i, rc = 0;
+	int i;
 	
 #ifdef ENABLE_NLS
 	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
@@ -122,72 +191,21 @@ int main (int argc, char **argv)
 	textdomain (GETTEXT_PACKAGE);
 #endif
 	
+	gc = g_ptr_array_new ();
+	srcdir = g_ptr_array_new ();
+	
+	for (i = 0; i < G_N_ELEMENTS (options); i++)
+		options[i].description = _(options[i].description);
+	
+	ctx = getopts_context_new (argc, argv, options, GETOPTS_FLAG_ALLOW_SINGLE_DASH | GETOPTS_FLAG_BREAK_ON_FIRST_NONARG);
+	getopts_parse_args (ctx);
+	args = getopts_get_args (ctx, NULL);
+	
 	program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Alleyoop"),
 				      GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
-				      GNOME_PARAM_POPT_FLAGS, POPT_CONTEXT_POSIXMEHARDER,
-				      GNOME_PARAM_POPT_TABLE, options,
 				      NULL);
 	
-	g_object_get ((GObject *) program, GNOME_PARAM_POPT_CONTEXT, &ctx, NULL);
-	
-	poptResetContext (ctx);
-	
-	gc = g_ptr_array_new ();
-	srcdir = g_ptr_array_new ();
-	while ((rc = poptGetNextOpt (ctx)) > 0) {
-		switch (rc) {
-		case 'v':
-			fprintf (stdout, "%s version %s\n", PACKAGE, VERSION);
-			
-			for (i = 0; i < gc->len; i++)
-				g_free (gc->pdata[i]);
-			g_ptr_array_free (gc, TRUE);
-			
-			g_ptr_array_free (srcdir, TRUE);
-			
-			poptFreeContext (ctx);
-			exit (0);
-		case 'I':
-			path = poptGetOptArg (ctx);
-			g_ptr_array_add (srcdir, (char *) path);
-			break;
-		case 'R':
-			path = poptGetOptArg (ctx);
-			g_ptr_array_add (srcdir, (char *) path);
-			add_subdirs (srcdir, gc, path);
-			break;
-		case 's':
-			name = poptGetOptArg (ctx);
-			if (!strcasecmp (name, "memcheck")) {
-				/* default */
-				tool = NULL;
-			} else if (!strcasecmp (name, "addrcheck")) {
-				tool = "addrcheck";
-			} else if (!strcasecmp (name, "cachegrind")) {
-				/*tool = "cachegrind";*/
-				fprintf (stderr, "%s is currently an unsupported tool\n", name);
-			} else if (!strcasecmp (name, "helgrind")) {
-				/*tool = "helgrind";*/
-				fprintf (stderr, "%s is currently an unsupported tool\n", name);
-			} else {
-				fprintf (stderr, "Unknown tool: %s\n", name);
-				
-				for (i = 0; i < gc->len; i++)
-					g_free (gc->pdata[i]);
-				g_ptr_array_free (gc, TRUE);
-				
-				g_ptr_array_free (srcdir, TRUE);
-				
-				poptFreeContext (ctx);
-				exit (0);
-			}
-			break;
-		}
-	}
-	
-	args = poptGetArgs (ctx);
-	
 	if ((srcdir_env = getenv ("ALLEYOOP_INCLUDE_PATH"))) {
 		/* add our environment to the list of srcdir paths */
 		char *path, *p;
@@ -217,17 +235,10 @@ int main (int argc, char **argv)
 	
 	gtk_main ();
 	
-	for (i = 0; i < gc->len; i++)
-		g_free (gc->pdata[i]);
-	g_ptr_array_free (gc, TRUE);
-	
-	g_ptr_array_free (srcdir, TRUE);
+	g_object_unref (program);
 	vg_strpool_shutdown ();
 	g_free (srcdir_env);
-	
-	g_object_unref (program);
-	
-	poptFreeContext (ctx);
+	shutdown ();
 	
 	return 0;
 }
