@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -279,18 +276,15 @@ symtab_resolve_addr (SymTab *symtab, void *addr, int demangle_cpp)
 		return NULL;
 	}
 	
-	if (lib->abfd->iostream == NULL) {
-		lib->abfd->iostream = (void *) fopen (lib->filename, "r+");
-		if (lib->abfd->iostream == NULL)
-			return NULL;
-	}
+	if (!lib->abfd->iostream && !(lib->abfd->iostream = (void *) fopen (lib->filename, "r+")))
+		return NULL;
 	
 	sym = g_new (SymTabSymbol, 1);
 	
-	addr -= (bfd_vma) lib->load_addr;
+	addr = (void *) ((size_t) addr - (size_t) lib->load_addr);
 	
 	if (bfd_find_nearest_line (lib->abfd, lib->text_section, lib->syms,
-				   (bfd_vma) addr - lib->text_section->vma,
+				   (bfd_vma) ((size_t) addr - (size_t) lib->text_section->vma),
 				   &sym->filename, &name, &sym->lineno)) {
 		if (name)
 			sym->function = demangle (lib->abfd, name, demangle_cpp);
