@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -1225,7 +1222,7 @@ static struct _MenuItem popup_menu_items[] = {
 	{ N_("Edit in GNU/Emacs"), NULL,     FALSE, FALSE, FALSE, FALSE, G_CALLBACK (emacs_cb),    STACK_MASK    },
 	{ N_("Edit in XEmacs"),    NULL,     FALSE, FALSE, FALSE, FALSE, G_CALLBACK (xemacs_cb),   STACK_MASK    },
 	{ N_("Edit in GVim"),      NULL,     FALSE, FALSE, FALSE, FALSE, G_CALLBACK (gvim_cb),     STACK_MASK    },
-	{ N_("Edit in Custom Editor"), NULL,     FALSE, FALSE, FALSE, FALSE, G_CALLBACK (custom_editor_cb), STACK_MASK },
+	{ N_("Edit in Custom Editor"), NULL, FALSE, FALSE, FALSE, FALSE, G_CALLBACK (custom_editor_cb), STACK_MASK },
 	MENU_ITEM_TERMINATOR
 };
 
@@ -1236,13 +1233,21 @@ tree_button_press (GtkWidget *treeview, GdkEventButton *event, gpointer user_dat
 	GtkTreeSelection *selection;
 	VgErrorStack *stack = NULL;
 	GtkTreeModel *model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	guint32 mask = 0;
 	GtkWidget *menu;
 	
+	if (!gtk_tree_view_get_path_at_pos ((GtkTreeView *) treeview, event->x, event->y, &path, NULL, NULL, NULL))
+		return;
+	
+	/* make sure the correct row is selected before continuing */
+	selection = gtk_tree_view_get_selection ((GtkTreeView *) treeview);
+	gtk_tree_selection_select_path (selection, path);
+	gtk_tree_path_free (path);
+	
 	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
 		/* right-click */
-		selection = gtk_tree_view_get_selection ((GtkTreeView *) treeview);
 		if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
 			gtk_tree_model_get ((GtkTreeModel *) model, &iter, COL_POINTER_STACK, &stack, -1);
 			if (stack == NULL)
