@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -37,13 +34,20 @@
 #include "vgstrpool.h"
 
 
-#define d(x)
+#define d(x) 
+#define w(x) x
 
+/* timestamp format used in Valgrind 2.4.0: "YY-MM-DD HH:MM:SS:MS" - based on date */
+#define VG_2_4_0_TIME_STAMP_FORMAT "%u-%.2u-%.2u %.2u:%.2u:%.2u.%.3u"
+
+/* timestamp format used in Valgrind 3.1.0: "DD:HH:MM:SS.MS" - based on time elapsed since program start */
+#define VG_3_1_0_TIME_STAMP_FORMAT "%.2u:%.2u:%.2u:%.2u.%.3u"
 
 enum {
 	VG_ERROR_PARSER_STATE_INIT,
 	VG_ERROR_PARSER_STATE_NEW_ERROR,
 	VG_ERROR_PARSER_STATE_PARTIAL_ERROR,
+	VG_ERROR_PARSER_STATE_WARNING = (1 << 8)
 };
 
 typedef struct _VgErrorListNode {
@@ -129,7 +133,7 @@ vg_error_set_state (VgErrorParser *parser, pid_t pid, int state)
 }
 
 static VgError *
-vg_error_new (VgErrorParser *parser, pid_t pid)
+vg_error_new (VgErrorParser *parser, pid_t pid, timestamp_t *stamp)
 {
 	VgErrorListNode *n;
 	VgError *err;
@@ -145,6 +149,7 @@ vg_error_new (VgErrorParser *parser, pid_t pid)
 	}
 	
 	err = g_new (VgError, 1);
+	memcpy (&err->stamp, stamp, sizeof (err->stamp));
 	err->pid = pid;
 	err->thread = (vgthread_t) -1;
 	err->summary = NULL;
@@ -229,7 +234,7 @@ vg_error_stack_new (VgErrorParser *parser, pid_t pid)
 	
 	if (!(n = g_hash_table_lookup (parser->pid_hash, GINT_TO_POINTER (pid)))) {
 		d(fprintf (stderr, "VgErrorParser appending stack frame to non-existant error report??\n"));
-		return;
+		return NULL;
 	}
 	
 	stack = g_new (VgErrorStack, 1);
@@ -249,13 +254,209 @@ vg_error_stack_new (VgErrorParser *parser, pid_t pid)
 	return stack;
 }
 
+static int
+decode_timestamp_uint (unsigned char **in, size_t inlen, unsigned char expect,
+		       int min, int max, const char *version, const char *token)
+{
+	register unsigned char *inptr = *in;
+	unsigned char *inend = *in + inlen;
+	int val = 0;
+	
+	while (inptr < inend && (*inptr >= '0' && *inptr <= '9')) {
+		if (((val * 10) + (*inptr - '0')) < val) {
+			val = -ERANGE;
+			break;
+		}
+		
+		val = (val * 10) + (*inptr - '0');
+		inptr++;
+	}
+	
+	if (inptr == *in) {
+		w(fprintf (stderr, "Expected Valgrind %s timestamp %s token: '%.*s'\n", version,
+			   token, inlen, *in));
+		return -1;
+	}
+	
+	if (!(val >= min && val <= max)) {
+#if w(!)0
+		if (val == -ERANGE) {
+			while (*inptr >= '0' && *inptr <= '9')
+				inptr++;
+		}
+		
+		fprintf (stderr, "Value out of range for Valgrind %s timestamp %s token: '%.*s'\n",
+			 version, token, inptr - *in, *in);
+#endif
+		return -1;
+	}
+	
+	if (*inptr != expect) {
+		w(fprintf (stderr, "Expected '%c' after Valgrind %s timestamp %s token: '%.*s'\n",
+			   expect, version, token, inend - inptr, inptr));
+		return -1;
+	}
+	
+	*in = inptr;
+	
+	return val;
+}
+
+static unsigned int
+decode_uint (unsigned char **in, size_t inlen)
+{
+	register unsigned char *inptr = *in;
+	unsigned char *inend = *in + inlen;
+	unsigned int val = 0;
+	
+	while (inptr < inend && (*inptr >= '0' && *inptr <= '9')) {
+		if (((val * 10) + (*inptr - '0')) < val) {
+			errno = ERANGE;
+			return 0;
+		}
+		
+		val = (val * 10) + (*inptr - '0');
+		inptr++;
+	}
+	
+	*in = inptr;
+	errno = 0;
+	
+	return val;
+}
+
+static gboolean
+decode_timestamp_and_pid (timestamp_t *stamp, pid_t *pid, unsigned char **in, size_t inlen)
+{
+	unsigned char *inend = *in + inlen;
+	unsigned char *inptr = *in;
+	unsigned char *start = *in;
+	unsigned int uval;
+	int val;
+	
+	/* extract a uint */
+	uval = decode_uint (&inptr, inlen);
+	if (inptr == *in) {
+		w(fprintf (stderr, "Expected pid or timestamp token: '%.*s'\n", inend - inptr, inptr));
+		return FALSE;
+	}
+	
+	switch (*inptr) {
+	case '-':
+		/* Valgrind 2.4.0 timestamp */
+		inptr = *in;
+		stamp->format = TIMESTAMP_FORMAT_2_4_0;
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), '-', 0, INT_MAX, "2.4.0", "year")) == -1)
+			return FALSE;
+		
+		stamp->ts_year = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), '-', 1, 12, "2.4.0", "month")) == -1)
+			return FALSE;
+		
+		stamp->ts_month = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ' ', 1, 31, "2.4.0", "day")) == -1)
+			return FALSE;
+		
+		stamp->ts_day = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ':', 0, 23, "2.4.0", "hour")) == -1)
+			return FALSE;
+		
+		stamp->ts_hour = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ':', 0, 59, "2.4.0", "minute")) == -1)
+			return FALSE;
+		
+		stamp->ts_min = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ':', 0, 59, "2.4.0", "second")) == -1)
+			return FALSE;
+		
+		stamp->ts_sec = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ' ', 0, 999, "2.4.0", "millisecond")) == -1)
+			return FALSE;
+		
+		stamp->ts_msec = val;
+		inptr++;
+		break;
+	case ':':
+		/* Valgrind 3.1.0 timestamp */
+		inptr = *in;
+		stamp->format = TIMESTAMP_FORMAT_3_1_0;
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ':', 0, INT_MAX, "3.1.0", "days")) == -1)
+			return FALSE;
+		
+		stamp->ts_days = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ':', 0, 23, "3.1.0", "hours")) == -1)
+			return FALSE;
+		
+		stamp->ts_hours = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ':', 0, 59, "3.1.0", "minutes")) == -1)
+			return FALSE;
+		
+		stamp->ts_mins = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), '.', 0, 59, "3.1.0", "seconds")) == -1)
+			return FALSE;
+		
+		stamp->ts_secs = val;
+		inptr++;
+		
+		if ((val = decode_timestamp_uint (&inptr, (inend - inptr), ' ', 0, 999, "3.1.0", "milliseconds")) == -1)
+			return FALSE;
+		
+		stamp->ts_msecs = val;
+		inptr++;
+		break;
+	default:
+		stamp->format = TIMESTAMP_FORMAT_NONE;
+		goto pid;
+	}
+	
+	start = inptr;
+	uval = decode_uint (&inptr, inend - inptr);
+	if (inptr == start) {
+		w(fprintf (stderr, "Expected pid token: '%.*s'\n", inend - inptr, inptr));
+		return FALSE;
+	}
+	
+ pid:
+	
+	if (uval == 0) {
+		w(fprintf (stderr, "Invalid pid received from valgrind: '%.*s'\n", inptr - start, start));
+		return FALSE;
+	}
+	
+	*pid = (pid_t) uval;
+	
+	*in = inptr;
+	
+	return TRUE;
+}
+
 int
 vg_error_parser_step (VgErrorParser *parser)
 {
 	register unsigned char *inptr;
 	unsigned char *start, *end;
+	timestamp_t stamp;
 	vgthread_t thread;
-	int ret, num = 0;
+	unsigned int num;
+	int state, ret;
 	VgError *err;
 	Parser *priv;
 	pid_t pid;
@@ -276,54 +477,77 @@ vg_error_parser_step (VgErrorParser *parser)
 		while (*inptr != '\n')
 			inptr++;
 		
-		d(fprintf (stderr, "parser_step: '%.*s'\n", inptr - start, start));
-		
 		if (inptr == priv->inend)
 			break;
 		
+		d(fprintf (stderr, "parser_step: '%.*s'\n", inptr - start, start));
+		
 		if (start[0] != '=' || start[1] != '=') {
-			d(fprintf (stderr, "Unexpected data received from valgrind: '%.*s'\n", inptr - start, start));
+			w(fprintf (stderr, "Unexpected data received from valgrind: '%.*s'\n", inptr - start, start));
 			inptr++;
 			start = inptr;
 			continue;
 		}
 		
 		start += 2;
-		if ((pid = strtoul (start, (char **) &end, 10)) == 0 || end == start || *end != '=') {
-			d(fprintf (stderr, "Invalid pid received from valgrind: '%.*s'\n", end - start, start));
+		
+		memset (&stamp, 0, sizeof (timestamp_t));
+		if (!decode_timestamp_and_pid (&stamp, &pid, &start, inptr - start)) {
 			inptr++;
 			start = inptr;
 			continue;
 		}
 		
-		if (end[0] != '=' || end[1] != '=') {
-			d(fprintf (stderr, "Unexpected data received from valgrind: '%.*s'\n", (inptr - start) + 2, start - 2));
+		if (start[0] != '=' || start[1] != '=' || start[2] != ' ') {
+			w(fprintf (stderr, "Unexpected data received from valgrind: '%.*s'\n", (inptr - start) + 2, start - 2));
 			inptr++;
 			start = inptr;
 			continue;
 		}
 		
-		start = end + 3;
+		start += 3;
 		
-		if (vg_error_get_state (parser, pid) != VG_ERROR_PARSER_STATE_PARTIAL_ERROR) {
+		*inptr = '\0';
+		if ((state = vg_error_get_state (parser, pid)) & VG_ERROR_PARSER_STATE_WARNING) {
+			/* eat indenting */
+			while (*start == ' ')
+				start++;
+			
+			if (strcmp (start, "your program may misbehave as a result") == 0) {
+				/* this marks the end of the Valgrind warning spewage */
+				vg_error_set_state (parser, pid, state & ~VG_ERROR_PARSER_STATE_WARNING);
+			}
+			
+			inptr++;
+			start = inptr;
+			continue;
+		}
+		
+		if (state != VG_ERROR_PARSER_STATE_PARTIAL_ERROR) {
 			/* brand new error - first line is the general report or thread-id (1.9.6 and later) */
-			err = vg_error_new (parser, pid);
+			err = vg_error_new (parser, pid, &stamp);
 			
 			if (start[0] == ' ') {
-				d(fprintf (stderr, "Unexpected SPACE received from valgrind: '%.*s'\n", inptr - start, start));
+				/* there should not be any additional error indenting here... */
+				w(fprintf (stderr, "Unexpected SPACE received from valgrind: '%.*s'\n", inptr - start, start));
 				
 				while (*start == ' ')
 					start++;
 			}
 			
-			if (start != inptr) {
-				*priv->inend = '\0';
+			if (start < inptr) {
 				if (strncmp (start, "discard syms in ", 16) == 0) {
+					/* "discard syms in /path/to/lib/libfoo.so" */
 					d(fprintf (stderr, "dropping 'discard syms in' spewage\n"));
 				} else if (strstr (start, "IGNORED call to:") != NULL) {
 					d(fprintf (stderr, "dropping ignored call notification\n"));
 				} else if (strstr (start, "KLUDGED call to:") != NULL) {
+					/* "valgrind's libpthread.so: KLUDGED call to: pthread_getschedparam" */
 					d(fprintf (stderr, "dropping kludged call notification\n"));
+				} else if (strncmp (start, "warning: ", 9) == 0) {
+					/* "warning: Valgrind's pthread_getschedparam is incomplete" */
+					d(fprintf (stderr, "dropping warning message\n"));
+					vg_error_set_state (parser, pid, state | VG_ERROR_PARSER_STATE_WARNING);
 				} else if (strncmp (start, "Thread ", 7) == 0) {
 					start += 7;
 					thread = strtoul (start, (char **) &end, 10);
@@ -345,13 +569,18 @@ vg_error_parser_step (VgErrorParser *parser)
 				start++;
 			
 			if (start < inptr) {
-				*priv->inend = '\0';
 				if (!strncmp (start, "discard syms in ", 16)) {
 					/* "discard syms in /path/to/lib/libfoo.so" */
 					d(fprintf (stderr, "dropping 'discard syms in' spew received from Valgrind\n"));
+				} else if (strstr (start, "IGNORED call to:") != NULL) {
+					d(fprintf (stderr, "dropping ignored call notification\n"));
 				} else if (strstr (start, "KLUDGED call to:") != NULL) {
 					/* "valgrind's libpthread.so: KLUDGED call to: pthread_getschedparam" */
 					d(fprintf (stderr, "dropping kludged call notification\n"));
+				} else if (strncmp (start, "warning: ", 9) == 0) {
+					/* "warning: Valgrind's pthread_getschedparam is incomplete" */
+					d(fprintf (stderr, "dropping warning message\n"));
+					vg_error_set_state (parser, pid, state | VG_ERROR_PARSER_STATE_WARNING);
 				} else if (strncmp (start, "at ", 3) != 0 && strncmp (start, "by ", 3) != 0) {
 					/* another summary report */
 					vg_error_summary_append (parser, pid, start, inptr - start);
@@ -533,15 +762,34 @@ vg_error_free (VgError *err)
 	g_free (err);
 }
 
+static void
+vg_error_timestamp_to_string (const timestamp_t *stamp, GString *str)
+{
+	switch (stamp->format) {
+	case TIMESTAMP_FORMAT_2_4_0:
+		g_string_append_printf (str, VG_2_4_0_TIME_STAMP_FORMAT " ", stamp->ts_year,
+					stamp->ts_month, stamp->ts_day, stamp->ts_hour,
+					stamp->ts_min, stamp->ts_sec, stamp->ts_msec);
+		break;
+	case TIMESTAMP_FORMAT_3_1_0:
+		g_string_append_printf (str, VG_3_1_0_TIME_STAMP_FORMAT " ", stamp->ts_days,
+					stamp->ts_hours, stamp->ts_mins, stamp->ts_secs,
+					stamp->ts_msecs);
+		break;
+	default:
+		break;
+	}
+}
 
 static void
 vg_error_stack_to_string (VgErrorStack *stack, GString *str)
 {
-	pid_t pid;
+	timestamp_t *stamp = &stack->summary->parent->stamp;
+	pid_t pid = stack->summary->parent->pid;
 	
-	pid = stack->summary->parent->pid;
-	
-	g_string_append_printf (str, "==%u==    %s ", pid, stack->where == VG_WHERE_AT ? "at" : "by");
+	g_string_append (str, "==");
+	vg_error_timestamp_to_string (stamp, str);
+	g_string_append_printf (str, "%u==    %s ", pid, stack->where == VG_WHERE_AT ? "at" : "by");
 	
 	if (stack->addr != VG_STACK_ADDR_UNKNOWN)
 		g_string_append_printf (str, "0x%.8x: ", stack->addr);
@@ -564,9 +812,12 @@ vg_error_stack_to_string (VgErrorStack *stack, GString *str)
 static void
 vg_error_summary_to_string (VgErrorSummary *summary, int indent, GString *str)
 {
+	timestamp_t *stamp = &summary->parent->stamp;
 	VgErrorStack *s;
 	
-	g_string_append_printf (str, "==%u== %s", summary->parent->pid, indent ? "   " : "");
+	g_string_append (str, "==");
+	vg_error_timestamp_to_string (stamp, str);
+	g_string_append_printf (str, "%u== %s", summary->parent->pid, indent ? "   " : "");
 	g_string_append (str, summary->report);
 	g_string_append_c (str, '\n');
 	
@@ -581,11 +832,15 @@ vg_error_summary_to_string (VgErrorSummary *summary, int indent, GString *str)
 void
 vg_error_to_string (VgError *err, GString *str)
 {
+	timestamp_t *stamp = &err->stamp;
 	VgErrorSummary *s;
 	int indent = 0;
 	
-	if (err->thread != (vgthread_t) -1)
-		g_string_append_printf (str, "==%u== Thread %d:\n", err->pid, err->thread);
+	if (err->thread != (vgthread_t) -1) {
+		g_string_append (str, "==");
+		vg_error_timestamp_to_string (stamp, str);
+		g_string_append_printf (str, "%u== Thread %d:\n", err->pid, err->thread);
+	}
 	
 	s = err->summary;
 	while (s != NULL) {
@@ -594,5 +849,7 @@ vg_error_to_string (VgError *err, GString *str)
 		s = s->next;
 	}
 	
-	g_string_append_printf (str, "==%u==\n", err->pid);
+	g_string_append (str, "==");
+	vg_error_timestamp_to_string (stamp, str);
+	g_string_append_printf (str, "%u==\n", err->pid);
 }
