@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -41,7 +38,9 @@
 #define DEMANGLE_KEY         "/apps/alleyoop/valgrind/general/demangle"
 #define NUM_CALLERS_KEY      "/apps/alleyoop/valgrind/general/num-callers"
 #define ERROR_LIMIT_KEY      "/apps/alleyoop/valgrind/general/error-limit"
+#ifdef SLOPPY_MALLOC
 #define SLOPPY_MALLOC_KEY    "/apps/alleyoop/valgrind/general/sloppy-malloc"
+#endif
 #define TRACE_CHILDREN_KEY   "/apps/alleyoop/valgrind/general/trace-children"
 #define TRACK_FDS_KEY        "/apps/alleyoop/valgrind/general/track-fds"
 #define TIME_STAMP_KEY       "/apps/alleyoop/valgrind/general/time-stamp"
@@ -203,6 +202,7 @@ vg_general_prefs_init (VgGeneralPrefs *prefs)
 	gtk_widget_show (widget);
 	gtk_box_pack_start ((GtkBox *) vbox, widget, FALSE, FALSE, 0);
 	
+#ifdef SLOPPY_MALLOC
 	bool = gconf_client_get_bool (gconf, SLOPPY_MALLOC_KEY, NULL);
 	widget = gtk_check_button_new_with_label (_("Round malloc sizes to next word"));
 	g_signal_connect (widget, "toggled", G_CALLBACK (toggle_button_toggled), SLOPPY_MALLOC_KEY);
@@ -210,6 +210,7 @@ vg_general_prefs_init (VgGeneralPrefs *prefs)
 	prefs->sloppy_malloc = (GtkToggleButton *) widget;
 	gtk_widget_show (widget);
 	gtk_box_pack_start ((GtkBox *) vbox, widget, FALSE, FALSE, 0);
+#endif
 	
 	bool = gconf_client_get_bool (gconf, TRACE_CHILDREN_KEY, NULL);
 	widget = gtk_check_button_new_with_label (_("Trace any child processes forked off by the program being debugged"));
@@ -314,7 +315,9 @@ static struct {
 	{ DEMANGLE_KEY,         "--demangle",         NULL, ARG_TYPE_BOOL,   1 },
 	{ NUM_CALLERS_KEY,      "--num-callers",      NULL, ARG_TYPE_INT,    4 },
 	{ ERROR_LIMIT_KEY,      "--error-limit",      NULL, ARG_TYPE_BOOL,   1 },
+#ifdef SLOPPY_MALLOC
 	{ SLOPPY_MALLOC_KEY,    "--sloppy-malloc",    NULL, ARG_TYPE_BOOL,   0 },
+#endif
 	{ TRACE_CHILDREN_KEY,   "--trace-children",   NULL, ARG_TYPE_BOOL,   0 },
 	{ TRACK_FDS_KEY,        "--track-fds",        NULL, ARG_TYPE_BOOL,   0 },
 	{ TIME_STAMP_KEY,       "--time-stamp",       NULL, ARG_TYPE_BOOL,   0 },
