@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -227,7 +224,7 @@ vg_rule_editor_init (VgRuleEditor *editor)
 	label = gtk_label_new (_("Rule name:"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	editor->name = (GtkEntry *) widget = gtk_entry_new ();
+	editor->name = (GtkEntry *) (widget = gtk_entry_new ());
 	gtk_widget_show (widget);
 	gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
 	gtk_widget_show (hbox);
@@ -237,7 +234,7 @@ vg_rule_editor_init (VgRuleEditor *editor)
 	label = gtk_label_new (_("Suppress messages of type:"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	editor->type = (GtkOptionMenu *) widget = rule_type_menu_new (editor);
+	editor->type = (GtkOptionMenu *) (widget = rule_type_menu_new (editor));
 	gtk_widget_show (widget);
 	gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
 	gtk_widget_show (hbox);
@@ -247,10 +244,10 @@ vg_rule_editor_init (VgRuleEditor *editor)
 	label = gtk_label_new (_("Suppress when using:"));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	editor->addrcheck = (GtkToggleButton *) widget = gtk_check_button_new_with_label ("Addrcheck");
+	editor->addrcheck = (GtkToggleButton *) (widget = gtk_check_button_new_with_label ("Addrcheck"));
 	gtk_widget_show (widget);
 	gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
-	editor->memcheck = (GtkToggleButton *) widget = gtk_check_button_new_with_label ("Memcheck");
+	editor->memcheck = (GtkToggleButton *) (widget = gtk_check_button_new_with_label ("Memcheck"));
 	gtk_widget_show (widget);
 	gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
@@ -261,7 +258,7 @@ vg_rule_editor_init (VgRuleEditor *editor)
 	gtk_misc_set_alignment ((GtkMisc *) label, 1.0, 0.5);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	editor->syscall = (GtkEntry *) widget = gtk_entry_new ();
+	editor->syscall = (GtkEntry *) (widget = gtk_entry_new ());
 	gtk_widget_show (widget);
 	gtk_widget_set_sensitive (widget, FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
@@ -276,7 +273,7 @@ vg_rule_editor_init (VgRuleEditor *editor)
 	/*editor->callers = g_ptr_array_new ();
 	  g_ptr_array_add (editor->callers, call_stack_frame_new (0, NULL));*/
 	
-	editor->call_stack = (GtkBox *) widget = call_stack_new (editor);
+	editor->call_stack = (GtkBox *) (widget = call_stack_new (editor));
 	gtk_widget_show (widget);
 	gtk_box_pack_start (GTK_BOX (vbox), widget, TRUE, TRUE, 0);
 }
