@@ -1,8 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Authors: Jeffrey Stedfast <fejj@ximian.com>
- *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+/*  Alleyoop
+ *  Copyright (C) 2003-2006 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
- *
  */
 
 
@@ -144,7 +141,7 @@ vg_rule_type_from_report (const char *report, vgrule_t *type, char **syscall)
 	} else if (!strcmp (report, "Invalid free() / delete / delete[]")) {
 		*type = VG_RULE_FREE;
 		return TRUE;
-	} else if ((inptr = strstr (report, " are still reachable in loss record "))) {
+	} else if ((inptr = strstr (report, " in loss record "))) {
 		*type = VG_RULE_LEAK;
 		return TRUE;
 	} else if (!strncmp (report, "Invalid read of size ", 21)) {
